package net.wanji.utc.config.security;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 业务认证中心 默认资源服务器配置
 * 提供默认resource、token、认证接口访问配置
 * 用户一般需要重写configure(HttpSecurity http)方法自定义受保护资源，
 *  如登录接口白名单或者swagger配置白名单
 * @Author mapabc
 * @Date 2020/4/12 21:01
 */
public class IdentityResourceServerConfig implements ResourceServerConfigurer {

    @Value("${security.oauth2.client.client-id}")
    private String resourceId;

    //私钥
    private final String SIGN_KEY = "dev";

    @Override
    public void configure(ResourceServerSecurityConfigurer resources) {
        //默认使用clientid 作为当前资源id
        resources.resourceId(resourceId).stateless(true);
        //jwt token 配置
        JwtAccessTokenConverter accessTokenConverter = new JwtAccessTokenConverter();
        accessTokenConverter.setSigningKey(SIGN_KEY);
        accessTokenConverter.setVerifierKey(SIGN_KEY);
        resources.tokenStore(new JwtTokenStore(accessTokenConverter));
        //自定义鉴权失败返回值
        resources.authenticationEntryPoint((request, response, authException) -> {
            response.setContentType("application/json");
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            Map<String, Object> result = new HashMap<>();
            if (StringUtils.contains(authException.getMessage(), "Access token expired")) {
                result.put("code", 407); //参见com.wanji.common.framework.HttpCode，作为基础模块，不单独引入依赖
                result.put("message", "token过期");
            } else {
                result.put("code", 401); //参见com.wanji.common.framework.HttpCode，作为基础模块，不单独引入依赖
                result.put("message", "鉴权失败");
            }
            result.put("status", "fail");
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.writeValue(response.getOutputStream(), result);
            } catch (Exception e) {
                throw new ServletException();
            }
        });
    }


    /**
     * @Description: 配置受保护资源，示例如下
     *
     *  http
     *                 .authorizeRequests()
     *                 .antMatchers(
     *                         "/webjars/**",
     *                         "/resources/**",
     *                         "/swagger-ui.html",
     *                         "/swagger-resources/**",
     *                         "/v2/api-docs",
     *                         "/getSwaggerToken",
     *                         "/bp/systemsetting/sysUser/login",
     *                 ).permitAll()
     *                 .anyRequest().authenticated()
     *                 .and()
     *                 .csrf().disable();
     *
     * @param http
     * @Author mapabc
     * @Date 2020/4/17 14:48
     **/
    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.authorizeRequests().anyRequest().authenticated();
    }
}
